@@ -8,6 +8,7 @@ import re
 import shutil
 import sys
 import tempfile
+import sysconfig
 import numpy as np
 
 from wand.api import library
@@ -17,6 +18,13 @@ from wand.image import Image
 import pcbnew
 from lxml import etree
 
+from pcbdraw import __version__
+
+STYLES_SUBDIR = 'styles'
+FOOTPRINTS_SUBDIR = 'footprints'
+PKG_BASE = os.path.dirname(__file__)
+data_path = [PKG_BASE]
+
 default_style = {
     "copper": "#417e5a",
     "board": "#4ca06c",
@@ -299,11 +307,11 @@ def get_board_polygon(svg_elements):
                 outline = [elements[0]]
                 elements = elements[1:]
             else:
-                e = etree.Element("path", d=path, style="fill-rule=evenodd;")
+                e = etree.Element("path", d=path, style="fill-rule: evenodd;")
                 return e
 
 def process_board_substrate_layer(container, name, source, colors, boardsize):
-    layer = etree.SubElement(container, "g", id="substrate-"+name,
+    layer = etree.SubElement(container, "g", id="substrate-" + name,
         style="fill:{0}; stroke:{0};".format(colors[name]))
     if name == "pads":
         layer.attrib["mask"] = "url(#pads-mask)"
@@ -374,10 +382,11 @@ def get_layers(board, colors, toPlot):
             pctl.SetLayer(l)
             pctl.PlotLayer()
     pctl.ClosePlot()
+    boardsize = board.ComputeBoundingBox()
     for f, _, process in toPlot:
         for svg_file in os.listdir(tmp):
             if svg_file.endswith("-" + f + ".svg"):
-                process(container, f, os.path.join(tmp, svg_file), colors)
+                process(container, f, os.path.join(tmp, svg_file), colors, boardsize)
     shutil.rmtree(tmp)
     return container
 
@@ -480,12 +489,11 @@ def get_hole_mask(board):
         if module.GetPadCount() == 0:
             module = module.Next()
             continue
-        pad = module.Pads()
         try:
-            pad.GetPosition()
-        except:
-            # Newest nightly renames Pads to PadList
             pad = module.PadsList()
+        except AttributeError:
+            # Older interfase used Pads()
+            pad = module.Pads()
         while pad:
             pos = pad.GetPosition()
             padOrientation = pad.GetOrientation()
@@ -528,7 +536,7 @@ def print_component(paths, lib, name, value, ref, pos, remapping={}):
         ref, lib, name, pos[0], pos[1], math.degrees(pos[2]), f if f else "Not found")
     print(msg)
 
-def component_from_library(lib, name, value, ref, pos, comp, highlight, silent):
+def component_from_library(lib, name, value, ref, pos, comp, highlight, silent, no_warn_back):
     if not name:
         return
     if comp["filter"] is not None and ref not in comp["filter"]:
@@ -536,7 +544,8 @@ def component_from_library(lib, name, value, ref, pos, comp, highlight, silent):
     f = get_model_file(comp["libraries"], lib, name, ref, comp["remapping"])
     if not f:
         if not silent:
-            print("Warning: component '{}' for footprint '{}' from library '{}' was not found".format(name, ref, lib))
+            if name[-5:] != '.back' or not no_warn_back:
+                print("Warning: component '{}' for footprint '{}' from library '{}' was not found".format(name, ref, lib))
         if comp["placeholder"]:
             etree.SubElement(comp["container"], "rect", x=str(ki2dmil(pos[0]) - 150), y=str(ki2dmil(pos[1]) - 150),
                              width="300", height="300", style="fill:red;")
@@ -583,24 +592,53 @@ def build_highlight(preset, width, height, pos, origin, ref):
         -math.degrees(pos[2]), -origin[0], -origin[1])
     h.attrib["id"] = "h_" + ref
 
-def svg_to_png(infile, outfile, dpi=300):
-    with Image(resolution=300) as image:
+def svg_to_bitmap(infile, outfile, dpi=300):
+    with Image(resolution=dpi) as image:
         with Color('transparent') as background_color:
             library.MagickSetBackgroundColor(image.wand,
                                             background_color.resource)
-        image.read(filename=infile, resolution=300)
-        png_image = image.make_blob("png32")
+        image.read(filename=infile, resolution=dpi)
+        _, ext = os.path.splitext(outfile)
+        if ext.lower() == ".png":
+            type = "png32"
+        elif ext.lower() in [".jpg", ".jpeg"]:
+            type = "jpeg"
+        else:
+            raise RuntimeError(f"Unsupported output image type {ext}")
+        bin_blob = image.make_blob(type)
         with open(outfile, "wb") as out:
-            out.write(png_image)
+            out.write(bin_blob)
+
+def find_data_file(name, ext, subdir):
+    if os.path.isfile(name):
+        return name
+    # Not a file here, needs extension?
+    ln = len(ext)
+    if name[-ln:] != ext:
+        name += ext
+        if os.path.isfile(name):
+            return name
+    # Try in the data path
+    for p in data_path:
+        fn = os.path.join(p, subdir, name)
+        if os.path.isfile(fn):
+            return fn
+    raise RuntimeError("Missing '" + subdir + "' " + name)
 
 def load_style(style_file):
+    if style_file.startswith("builtin:"):
+        STYLES = os.path.join(PKG_BASE, "styles")
+        style_file = os.path.join(STYLES, style_file[len("builtin:"):])
+    else:
+        style_file = find_data_file(style_file, '.json', STYLES_SUBDIR)
     try:
         with open(style_file, "r") as f:
             style = json.load(f)
     except IOError:
         raise RuntimeError("Cannot open style " + style_file)
     required = set(["copper", "board", "clad", "silk", "pads", "outline",
-        "highlight-style", "highlight-offset", "highlight-on-top"])
+        "vcut", "highlight-style", "highlight-offset", "highlight-on-top",
+        "highlight-padding"])
     missing = required - set(style.keys())
     if missing:
         raise RuntimeError("Missing following keys in style {}: {}"
@@ -621,15 +659,43 @@ def load_remapping(remap_file):
         raise RuntimeError("Cannot open remapping file " + remap_file)
 
 def adjust_lib_path(path):
-    base = os.path.dirname(__file__)
     if path == "default" or path == "kicad-default":
-        return os.path.join(base, "footprints", "KiCAD-base")
+        return [os.path.join(p, FOOTPRINTS_SUBDIR, "KiCAD-base") for p in data_path]
     if path == "eagle-default":
-        return os.path.join(base, "footprints", "Eagle-export")
-    return path
+        return [os.path.join(p, FOOTPRINTS_SUBDIR, "Eagle-export") for p in data_path]
+    return [path]
+
+def setup_data_path():
+    global data_path
+    share = os.path.join('share', 'pcbdraw')
+    entries = len(data_path)
+    scheme_names = sysconfig.get_scheme_names()
+    if os.name == 'posix':
+        if 'posix_user' in scheme_names:
+            data_path.append(os.path.join(sysconfig.get_path('data', 'posix_user'), share))
+        if 'posix_prefix' in scheme_names:
+            data_path.append(os.path.join(sysconfig.get_path('data', 'posix_prefix'), share))
+    elif os.name == 'nt':
+        if 'nt_user' in scheme_names:
+            data_path.append(os.path.join(sysconfig.get_path('data', 'nt_user'), share))
+        if 'nt' in scheme_names:
+            data_path.append(os.path.join(sysconfig.get_path('data', 'nt'), share))
+    if len(data_path) == entries:
+        data_path.append(os.path.join(sysconfig.get_path('data'), share))
 
 def main():
-    parser = argparse.ArgumentParser()
+    setup_data_path()
+    epilog = "Searching for styles on: "
+    c = len(data_path)
+    for i, path in enumerate(data_path):
+        epilog += "'"+os.path.join(path, 'styles')+"'"
+        if i == c-2:
+            epilog += " and "
+        elif i != c-1:
+            epilog += ", "
+
+    parser = argparse.ArgumentParser(epilog=epilog)
+    parser.add_argument("--version", action="version", version=f"PcbDraw {__version__}")
     parser.add_argument("-s", "--style", help="JSON file with board style")
     parser.add_argument("board", help=".kicad_pcb file to draw")
     parser.add_argument("output", help="destination for final SVG or PNG file")
@@ -647,9 +713,14 @@ def main():
     parser.add_argument("-f", "--filter", help="comma separated list of components to show")
     parser.add_argument("-v", "--vcuts", action="store_true", help="Render V-CUTS on the Cmts.User layer")
     parser.add_argument("--silent", action="store_true", help="Silent warning messages about missing footprints")
+    parser.add_argument("--dpi", help="DPI for bitmap output", type=int, default=300)
+    parser.add_argument("--no-warn-back", action="store_true", help="Don't show warnings about back footprints")
 
     args = parser.parse_args()
-    args.libs = [adjust_lib_path(path) for path in args.libs.split(',')]
+    libs = []
+    for path in args.libs.split(','):
+        libs.extend(adjust_lib_path(path))
+    args.libs = libs
     args.highlight = args.highlight.split(',') if args.highlight is not None else []
     args.filter = args.filter.split(',') if args.filter is not None else None
 
@@ -663,8 +734,8 @@ def main():
         print(e)
         sys.exit(1)
 
-    if os.path.splitext(args.output)[-1].lower() not in [".svg", ".png"]:
-        print("Output can be either an SVG or PNG file")
+    if os.path.splitext(args.output)[-1].lower() not in [".svg", ".png", ".jpg", ".jpeg"]:
+        print("Output can be either an SVG, PNG or JPG file")
         sys.exit(1)
 
     try:
@@ -722,12 +793,12 @@ def main():
         board_cont.append(get_layers(board, style, [("vcut", [pcbnew.Cmts_User], process_board_substrate_layer)]))
 
     walk_components(board, args.back, lambda lib, name, val, ref, pos:
-        component_from_library(lib, name, val, ref, pos, components, highlight, args.silent))
+        component_from_library(lib, name, val, ref, pos, components, highlight, args.silent, args.no_warn_back))
 
     # make another pass for search, and if found, render the back side of the component
     # the function will search for file with extension ".back.svg"
     walk_components(board, not args.back, lambda lib, name, val, ref, pos:
-        component_from_library(lib, name+".back", val, ref, pos, components, highlight, args.silent))
+        component_from_library(lib, name+".back", val, ref, pos, components, highlight, args.silent, args.no_warn_back))
 
     if args.output.endswith(".svg") or args.output.endswith(".SVG"):
         document.write(args.output)
@@ -735,7 +806,7 @@ def main():
         with tempfile.NamedTemporaryFile(suffix=".svg") as tmp_f:
             document.write(tmp_f)
             tmp_f.flush()
-            svg_to_png(tmp_f.name, args.output)
+            svg_to_bitmap(tmp_f.name, args.output, dpi=args.dpi)
 
 if __name__ == '__main__':
     main()
\ No newline at end of file
